-------------------------------------------------------------------------------------
-- | Namespace Definitions |---------------------------------------------------------
-------------------------------------------------------------------------------------

-- Retrieve the addon name and the namespace table.
local addon, npcg = ...

local functua = npcg.functua

local list = { }

functua.list = list

-------------------------------------------------------------------------------------
-- | Type Definitions | -------------------------------------------------------------
-------------------------------------------------------------------------------------

-- type List a :: { int, a }
-- A list is a secuence of items of which iteration is ordered (by key) and does not provide keys.

-- type Action a :: (a -> ())
-- An action is a function that takes "a" as an argument, and returns void.

-------------------------------------------------------------------------------------
-- | Iteration Functions | ----------------------------------------------------------
-------------------------------------------------------------------------------------

-- forEach :: Action a -> List a -> ()
-- Performs the same action on every element in a list.
function list.forEach(a, xs)
	for _,x in ipairs(xs) do
		a(x)
	end
end

-- map :: (a -> b) -> List a -> List b
-- Applies a function to every item in a table and returns the result.
-- Ordering is preserved.
function list.map(f, xs)
	local output = { }
	for i,v in ipairs(xs) do
		output[i] = f(v)
	end
	return output
end

-- filter :: (a -> bool) -> List a -> List a
-- Returns a list of all elements in a list that satisfy a predicate.
function list.filter(p, xs)
	local output = { }
	for _,v in ipairs(xs) do
		if p(v) then 
			table.insert(output, v)
		end
	end
	return output
end

-- foldr :: (a -> b -> b) -> b -> List a -> b
-- Applies an accumulator function on every element of a given list.
function list.foldr(op, val, xs)
	for _,v in ipairs(xs) do
		val = op(val, v)
	end
	return val
end

-------------------------------------------------------------------------------------
-- | Combination Functions | --------------------------------------------------------
-------------------------------------------------------------------------------------

-- concat :: List (List a) -> List a
-- Concatenates a list of lists.
function list.concat(xs)
	local output = { }
	for _,v in ipairs(xs) do
		for _,vv in ipairs(v) do
			table.insert(output, vv)
		end
	end
	return output
end

-------------------------------------------------------------------------------------
-- | Cloning Functions | ------------------------------------------------------------
-------------------------------------------------------------------------------------

-- clone :: List a -> List a
-- Returns a shallow clone of a table.
function list.clone(xs)
	return list.map(functua.id, xs)
end



-- End of File.